#include "storedip"

struct ClientData {
    int targetbackend;
    time_t lastaccess;
};

struct ClientDataCmp {
    bool operator() (struct in_addr a, struct in_addr b) const {
	long la, lb;
	memcpy (&la, &a, sizeof(long));
	memcpy (&lb, &b, sizeof(long));
	return (la - lb) < 0;
    }
};

static map<struct in_addr, ClientData, ClientDataCmp> store;

unsigned StoredIp::target(struct in_addr clientip,
			  BackendVector const &targetlist) {
    unsigned target;
    time_t now = time(0);

    // Is the client already known in the map, and not timed out?
    map<struct in_addr, ClientData, ClientDataCmp>::iterator hit =
	store.find(clientip);
    if (store.count(clientip) > 0) {
	time_t diff = now - store[clientip].lastaccess;
	
	msg ("Client IP " + static_cast<string>(inet_ntoa(clientip)) +
	     (Mstr(" last visited on ")
	      + timestamp(store[clientip].lastaccess)) +
	     (Mstr(", ") + diff) +
	     " sec ago, and went to " +
	     balancer.backend(store[clientip].targetbackend).description() +
	     "\n");
	
	if (diff <= config.ipstoretimeout()) {
	    // Recent 'nuff
	    target = store[clientip].targetbackend;
	    if (! balancer.backend(target).available()) {
		// Historical target down - get new one if in lax mode
		if (config.dispatchmode() == Dispatchmode::m_strict_stored_ip)
		    throw static_cast<Error>
			("Stored-IP algorithm: target back end ") +
			balancer.backend(target).description() +
			"unavailable";
		else {
		    msg ("Stored IP algorithm: target back end " +
			 balancer.backend(target).description()  +
			 " unavailable, falling back to least-connections\n");
		    Leastconn l;
		    target = l.target(clientip, targetlist);
		}
	    }
	} else {
	    // Not recent anymore
	    msg ("Visit too long ago, re-dispatching with least-connections\n");
	    Leastconn l;
	    target = l.target(clientip, targetlist);
	}
    } else {
	// Historical target unknown, fetch new one
	msg ("New visit from " + static_cast<string>(inet_ntoa(clientip)) +
	     "\n");
	Leastconn l;
	target = l.target(clientip, targetlist);
    }

    // Update the info.
    ClientData entry = {target, now};
    store[clientip] = entry;

    // Weed out store
    map<struct in_addr, ClientData, ClientDataCmp>::iterator
	iter = store.begin();
    while (iter != store.end()) {
	debugmsg ("Stored-IP: " +
		  (string)inet_ntoa((*iter).first) + " visited on " +
		  timestamp((*iter).second.lastaccess) + "\n");
	if (now - ((*iter).second.lastaccess) > config.ipstoretimeout())
	    store.erase(iter);
	else
	    iter++;
    }

    // Return target to caller
    return (target);
}
