#include "thread"
#include "profiler/profiler"

static void *_run (void *data) {
    Thread *t = (Thread*) data;

    try {
	t->execute();
    } catch (Error const &e) {
	cerr << e.what() << "\n";
    }

    // Cleanups
    delete (t);

    // To satisfy the prototype
    return (0);
}

void Thread::start() {
    PROFILE ("Thread::start");
    
    if (config.foregroundmode())
	_run ((void*)this);
    else {
	pthread_t th;
	pthread_attr_t attr;
	int res;

	if (pthread_attr_init (&attr))
	    throw static_cast<Error>("Cannot initialize thread attributes");
	if (pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED))
	    throw static_cast<Error>("Cannot set thread state as detached");
	for (int i = 0; i < 3; i++) {
	    res = pthread_create (&th, &attr, _run, (void*) this);
	    if (!res) {
		pthread_attr_destroy (&attr);
		return;
	    } else if (res == EAGAIN) {
		if (config.verbose())
		    msg ("Failed to start thread: " + (string)strerror(res) +
			 ", retrying\n");
		sleep (1);
		continue;
	    } else  {
		pthread_attr_destroy (&attr);
		throw static_cast<Error>("Failed to start thread: ") +
		    strerror(res);
	    }
	}
		
	throw static_cast<Error>("Failed to start thread: "
				 "Resources unavailable after 3 tries, "
				 "giving up");
    }
}
