<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html"/>

<xsl:template match="/">
  <html>
    <head>
      <title>XR Status Overview</title>
      <style type="text/css">
	h1 {
	  font-family: Verdana,Helvetica;
	  font-size: 12pt;
	  color: blue;
	}
	body {
	  font-family: Verdana,Helvetica;
	  font-size: 10pt;
	}
	td {
	  font-family: Verdana,Helvetica;
	  font-size: 8pt;
	  background-color: #ffff99; }
	}
	input {
	  font-family: Verdana,Helvetica;
	  font-size: 8pt;
	  background-color: #ffff99; }
	}
	.server  { background-color: #f3f399; }
	.backend { background-color: #f3f099; }
	.footer  { color: gray; }
      </style>
      <script type="text/javascript">
	function goto(uri, input) {
	  if (input == '')
	    document.location = uri;
	  else {
	    var el = document.getElementById(input);
	    if (el) {
	      var value = el.value;
	      if (value != "")
	        document.location = uri + encodeURIComponent(value);
	      else
	        document.location = uri;
	    }
	  }
	}
      </script>
    </head>
    <body>
      <h1>XR Status Overview</h1>
      <hr/>
      <xsl:apply-templates/>
    </body>
  </html>
</xsl:template>

<xsl:template match="/status">
  <table>
    <xsl:apply-templates/>
  </table>
</xsl:template>

<xsl:template match="/status/server">
  <tr>
    <td class="server" colspan="3">
      <b>Server <xsl:value-of select="address"/> </b>
    </td>
    <td>
      <input type="button" onclick="goto('/', '');" value="Refresh"/>
    </td>
  </tr>
  <tr>
    <td>Type</td>
    <td colspan="3"> <xsl:value-of select="type"/> </td>
  </tr>
  <tr>
    <td>Dispatch mode</td>
    <td colspan="3"> <xsl:value-of select="dispatchmode"/> </td>
  </tr>
  <tr>
    <td>Checks</td>
    <td>Wakeup interval</td>
    <td>
      <xsl:choose>
	<xsl:when test="checks/wakeupinterval = 0">
	  off
	</xsl:when>
	<xsl:otherwise>
	  sec
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="wakeupinterval"
	     id="wakeupinterval" value="{checks/wakeupinterval}"
	     onchange="goto('/server/wakeupinterval/', 'wakeupinterval');"/>
    </td>
  </tr>
  <tr>
    <td></td>
    <td>Checkup interval</td>
    <td>
      <xsl:choose>
	<xsl:when test="checks/checkupinterval = 0">
	  off
	</xsl:when>
	<xsl:otherwise>
	  sec
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="checkupinterval"
	     id="checkupinterval" value="{checks/checkupinterval}"
	     onchange="goto('/server/checkupinterval/', 'checkupinterval');"/>
    </td>
  </tr>
  <tr>
    <td>Timeouts</td>
    <td>Client</td>
    <td>
      <xsl:choose>
	<xsl:when test="clienttimeout = 0">
	  unlimited
	</xsl:when>
	<xsl:otherwise>
	  sec
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="clienttimeout"
	     id="clienttimeout" value="{clienttimeout}"
	     onchange="goto('/server/clienttimeout/', 'clienttimeout');"/>
    </td>
  </tr>
  <tr>
    <td></td>
    <td>Backend</td>
    <td>
      <xsl:choose>
	<xsl:when test="backendtimeout = 0">
	  unlimited
	</xsl:when>
	<xsl:otherwise>
	  sec
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="backendtimeout"
	     id="backendtimeout" value="{backendtimeout}"
	     onchange="goto('/server/backendtimeout/', 'backendtimeout');"/>
    </td>
  </tr>
 <tr>
    <td>Debugging</td>
    <td colspan="2">Verbose logging</td>
    <td>
      <xsl:choose>
	<xsl:when test="debugging/verbose = 0">
	  <select onchange="goto('/server/verbose/on', '');">
	    <option value="yes">yes</option>
	    <option value="no" selected="1">no</option>
	  </select>
	</xsl:when>
	<xsl:otherwise>
	  <select onchange="goto('/server/verbose/off', '');">
	    <option value="yes" selected="1">yes</option>
	    <option value="no">no</option>
	  </select>
	</xsl:otherwise>
      </xsl:choose>
    </td>
  </tr>
  <tr>
    <td></td>
    <td colspan="2">Debug logging</td>
    <td>
      <xsl:choose>
	<xsl:when test="debugging/debug = 0">
	  <select onchange="goto('/server/debug/on', '');">
	    <option value="yes">yes</option>
	    <option value="no" selected="1">no</option>
	  </select>
	</xsl:when>
	<xsl:otherwise>
	  <select onchange="goto('/server/debug/off', '');">
	    <option value="yes" selected="1">yes</option>
	    <option value="no">no</option>
	  </select>
	</xsl:otherwise>
      </xsl:choose>
    </td>
  </tr>
  <tr>
    <td></td>
    <td colspan="2">Traffic log directory</td>
    <td>
      <input type="text" size="8" name="logtrafficdir" id="logtrafficdir"
	     value="{debugging/logtrafficdir}"
	     onchange="goto('/server/logtrafficdir/', 'logtrafficdir');"/>
    </td>
  </tr>
  <tr>
    <td>Network buffer size</td>
    <td colspan="2">bytes</td>
    <td>
      <input type="text" size="8" name="serverbufsz" id="serverbufsz"
	     value="{buffersize}"
	     onchange="goto('/server/buffersize/', 'serverbufsz');"/>
    </td>
  </tr>
  <tr>
    <td> Max. connections </td>
    <td colspan="2">
      <xsl:choose>
	<xsl:when test="maxconnections = 0">
	  unlimited
	</xsl:when>
	<xsl:otherwise>
	  maximum value (0 for unlimited)
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="setservermaxcon" class="input"
	     id="setservermaxcon" value="{maxconnections}"
	     onchange="goto('/server/maxconnections/', 'setservermaxcon');"/>
    </td>
  </tr>
  <xsl:if test="/status/server/type = 'http'">
    <xsl:apply-templates select="/status/server/http"/>
  </xsl:if>
</xsl:template>

<xsl:template match="/status/backend">
  <tr> <td colspan="4"><hr/></td></tr>
  <tr>
    <td class="backend" colspan="4">
      <b> Back end <xsl:value-of select="address"/> </b>
    </td>
  </tr>
  <tr>
    <td><b>State</b></td>
    <td>health</td>
    <td colspan="2">
      <xsl:value-of select="live"/>,
      <xsl:value-of select="available"/>
    </td>
  </tr>
  <tr>
    <td></td>
    <td>connections</td>
    <td colspan="2"><xsl:value-of select="connections"/></td>
  </tr>
  <tr>
    <td></td>
    <td>served</td>
    <td colspan="2">
      <xsl:value-of select="bytesserved"/> bytes,
      <xsl:value-of select="clientsserved"/> clients
    </td>
  </tr>
  <tr>
    <td><b>Options</b></td>
    <td colspan="2">weight</td>
    <td>
      <input type="text" size="8" name="setbackendweight{nr}"
	     id="setbackendweight{nr}" value="{weight}"
	     onchange="goto('/backend/{nr}/weight/', 'setbackendweight{nr}');"/>
    </td>
  </tr>
  <tr>
    <td></td>
    <td>max. connections</td>
    <td>
      <xsl:choose>
	<xsl:when test="maxconnections = 0">
	  unlimited
	</xsl:when>
	<xsl:otherwise>
	  maximum value (0 for unlimited)
	</xsl:otherwise>
      </xsl:choose>
    </td>
    <td>
      <input type="text" size="8" name="setbackendmaxcon{nr}" class="input"
	     id="setbackendmaxcon{nr}" value="{maxconnections}"
	     onchange="goto('/backend/{nr}/maxconnections/', 'setbackendmaxcon{nr}');"/>
    </td>
  </tr>
  <tr>
    <td></td>
    <td colspan="2">load average</td>
    <td>
      <input type="text" size="8" name="setloadaverage{nr}"
	     id="setloadaverage{nr}" value="{loadavg}"
	     onchange="goto('/backend/{nr}/loadavg/', 'setloadaverage{nr}');"/>
    </td>
  </tr>
  <xsl:if test="/status/server/type = 'http'">
    <tr>
      <td></td>
      <td>host match</td>
      <td>
	<xsl:choose>
	  <xsl:when test="hostmatch = '.'">
	    any host request
	  </xsl:when>
	  <xsl:otherwise>
	    (. for any host)
	  </xsl:otherwise>
	</xsl:choose>
      </td>
      <td>
	<input type="text" size="8" name="sethostmatch{nr}"
	       id="sethostmatch{nr}" value="{hostmatch}"
	       onchange="goto('/backend/{nr}/hostmatch/', 'sethostmatch{nr}');"/>
	</td>
    </tr>
  </xsl:if>
  <tr>
    <td></td>
    <td colspan="2">Up state</td>
    <td>
      <xsl:choose>
	<xsl:when test="up = 'up'">
	  <select onchange="goto('/backend/{nr}/up/off', '');">
	    <option value="yes" selected="1">yes</option>
	    <option value="no">no</option>
	  </select>
	</xsl:when>
	<xsl:otherwise>
	  <select onchange="goto('/backend/{nr}/up/on', '');">
	    <option value="yes">yes</option>
	    <option value="no" selected="1">no</option>
	  </select>
	</xsl:otherwise>
      </xsl:choose>
    </td>
  </tr>
</xsl:template>

<xsl:template match="/status/server/http">
  <tr>
    <td>HTTP Goodies</td>
    <td colspan="2">add X-Forwarded-For</td>
    <td>
      <xsl:choose>
	<xsl:when test="addxforwardedfor = 0">
	  <select onchange="goto('/server/addxforwardedfor/on', '');">
	    <option value="yes">yes</option>
	    <option value="no" selected="1">no</option>
	  </select>
	</xsl:when>
	<xsl:otherwise>
	  <select onchange="goto('/server/addxforwardedfor/off', '');">
	    <option value="yes" selected="1">yes</option>
	    <option value="no">no</option>
	  </select>
	</xsl:otherwise>
      </xsl:choose>
    </td>
  </tr>
  <tr>
    <td></td>
    <td colspan="2">sticky HTTP</td>
    <td>
      <xsl:choose>
	<xsl:when test="stickyhttp = 0">
	  <select onchange="goto('/server/stickyhttp/on', '');">
	    <option value="yes">yes</option>
	    <option value="no" selected="1">no</option>
	  </select>
	</xsl:when>
	<xsl:otherwise>
	  <select onchange="goto('/server/stickyhttp/off', '');">
	    <option value="yes" selected="1">yes</option>
	    <option value="no">no</option>
	  </select>
	</xsl:otherwise>
      </xsl:choose>
    </td>
  </tr>
  <xsl:apply-templates select="/status/server/http/serverheaders"/>
</xsl:template>

<xsl:template match="/status/server/http/serverheaders">
  <xsl:for-each select="serverheader">
    <tr>
      <td></td>
      <td colspan="2">server header (. to delete)</td>
      <td>
	<input type="text" size="8" name="serverheader{nr}"
	       id="serverheader{nr}" value="{header}"
	       onchange="goto('/server/changeheader/{nr}/', 'serverheader{nr}');"/>
      </td>
    </tr>
  </xsl:for-each>
  <tr>
    <td></td>
    <td colspan="2">new server header</td>
    <td>
      <input type="text" size="8" name="newserverheader"
	     id="newserverheader"
	     onchange="goto('/server/newheader/', 'newserverheader');"/>
    </td>
  </tr>
</xsl:template>

<xsl:template match="*"/>

</xsl:stylesheet>

