#include "fdset"

int Fdset::readwriteable() const {
    PROFILE("Fdset::readwriteable");
    
    fd_set readset, writeset, exceptset;
    struct timeval tv, *tvp;

    // No fd's? Nothing is readable.
    if (set.size() < 1)
	return (-1);

    // Prepare select sets.
    FD_ZERO (&readset);
    FD_ZERO (&writeset);
    FD_ZERO (&exceptset);
    for (unsigned i = 0; i < set.size(); i++) {
	FD_SET (set[i], &readset);
	FD_SET (set[i], &writeset);
	FD_SET (set[i], &exceptset);
    }

    // Prepare timout specifier.
    if (tsec) {
	tv.tv_sec = tsec;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    // Run the select.
    if (select (FD_SETSIZE, &readset, &writeset, &exceptset, tvp) < 0) {
	if (errno != EINTR)
	    throw static_cast<Error>
		("Select failure: failed to wait for read/writeablestate: ")
		+ strerror(errno);
	else
	    return (-1);
    }

    // Check for exceptions.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &exceptset))
	    throw static_cast<Error>("Exception on fd/socket ") + set[i];

    // Check what's active.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &readset) && FD_ISSET (set[i], &writeset))
	    return (set[i]);

    // Nothing..
    return (-1);
}
    
