#include "fdset"

int Fdset::writeable() const {
    PROFILE("Fdset::writeable");
    
    fd_set writeset, exceptset;
    struct timeval tv, *tvp;

    // No fd's? Nothing is writeable.
    if (set.size() < 1)
	return (-1);

    // Prepare select sets.
    FD_ZERO (&writeset);
    FD_ZERO (&exceptset);
    for (unsigned i = 0; i < set.size(); i++) {
	FD_SET (set[i], &writeset);
	FD_SET (set[i], &exceptset);
    }

    // Prepare timout specifier.
    if (tsec) {
	tv.tv_sec = tsec;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    // Run the select.
    if (select (FD_SETSIZE, 0, &writeset, &exceptset, tvp) < 0) {
	if (errno != EINTR)
	    throw static_cast<Error>
		("Select failure: failed to wait for writable state: ") +
		strerror(errno);
	else
	    return (-1);
    }

    // Check for exceptions.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &exceptset))
	    throw static_cast<Error>("Exception on fd/socket ") + set[i];

    // Check what's writeable.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &writeset)) {
	    debugmsg (Mstr("Fd ") + set[i] + " has become writeable\n");
	    return (set[i]);
	}
    
    // Nothing..
    debugmsg ("No writeable fd's at this time\n");
    return (-1);
}
    
