#include "httpdispatcher"

void HttpDispatcher::dispatch() {
    PROFILE("HttpDispatcher::dispatch");
    
    unsigned stickytarget;
    string host_header = "";

    // Try to dispatch. Since we're in HTTP mode, we must return an
    // error page when dispatching fails.

    try {

	// Get the client's request. May need for cookie inspection or for the
	// host header.
	while (!buf.headersreceived())
	    if (!buf.netread(clientfd(), config.client_timeout()))
		throw static_cast<Error>("Didn't receive a valid "
					 "client request.\n");
	if (config.verbose())
	    msg ("Received client request: '" + buf.firstline() +
		 "'\n");

	// See if hostmatching is used. This is true when a backend
	// matches against a non-dot host.
	bool hostmatchused = false;
	for (unsigned i = 0; i < balancer.nbackends(); i++)
	    if (balancer.backend(i).hostmatch() != ".") {
		hostmatchused = true;
		break;
	    }
	// Build new target list if host matching applies.
	if (hostmatchused) {
	    host_header = buf.headerval ("Host");
	    if (config.verbose())
		msg ("Will try to dispatch request host '" +
		     host_header + "'\n");

	    // We need to build tcpdispatcher's target list now!
	    // Construct locally and poke into TcpDispatcher.
	    msg ("Creating host-based target list for the HTTP dispatcher\n");
	    BackendVector v;
	    v.isdefined(true);
	    for (unsigned i = 0; i < balancer.nbackends(); i++) {
		if ( (balancer.backend(i).available()) &&
		     (!regexec (&(balancer.backend(i).hostregex()),
				host_header.c_str(), 0, 0, 0)) ) {
		    v.add(i);
		    if (config.verbose())
			msg ("  Candidate target: " +
			     balancer.backend(i).description() + "\n");
		}
	    }
	    targetlist(v);
	}

	// Dispatch as a normal backend if sticky HTTP is off, or if the
	// sticky target is badly specified.
	if (!config.stickyhttp() ||
	    sscanf (buf.cookievalue ("XRTarget").c_str(),
		    "%d", &stickytarget) < 1 ||
	    stickytarget >= balancer.nbackends()) {
	    issticky(false);
	    TcpDispatcher::dispatch();
	} else {
	    // Got a sticky target. Try to connect. If that fails, fallback
	    // to non-sticky dispatching.
	    targetbackend(stickytarget);
	    Backend tb = balancer.backend(stickytarget);
	    if (config.verbose())
		msg ("Sticky HTTP request for " + tb.description() + "\n");
	    if (! tb.connect()) {
		balancer.backend(stickytarget).live(false);
		if (config.verbose())
		    msg ("Failed to connect to back end " + tb.description() +
			 ", trying to dispatch to other\n");
		issticky(false);
		TcpDispatcher::dispatch();
	    } else {
		backendfd(tb.sock());
		issticky(true);
	    }
	}
	
    } catch (Error const &e) {
	senderrorpage();
	throw e;
    }
	    
}

