#include "httpdispatcher"

void HttpDispatcher::handle() {
    PROFILE("HttpDispatcher::handle");

    // The client request was already retrieved before starting the
    // dispatcher. We can continue by applying server-directed headers.
    debugmsg("Applying server-directed headers to client request\n");
    buf.setversion('0');
    buf.setheader ("Connection", "close");
    buf.setheader ("Proxy-Connection", "close");

    // Apply other server-side directed info
    if (config.addxrversion())
	buf.setheader ("XR", VER);
    if (config.addxforwardedfor())
	buf.addheader ("X-Forwarded-For", string(inet_ntoa(clientip())));
    for (unsigned n = 0; n < config.nserverheaders(); n++)
	buf.addheader (config.serverheader(n));

    // Flush client info received so far to the back end.
    debugmsg("Sending client request to back end\n");
    buf.netwrite(backendfd(), config.backend_timeout());

    // Let's see if we need to modify the server headers.
    bool modify_serverheaders = false;
    if (config.addxrversion() ||
	(config.stickyhttp() && !issticky()))
	modify_serverheaders = true;

    // Go into copy-thru mode. If required, catch the server headers on
    // their first appearance and modify them.
    while (1) {
	Fdset readset (config.client_timeout());
	readset.add(clientfd());
	readset.add(backendfd());

	int sock;
	if ((sock = readset.readable()) < 0)
	    break;
	
	buf.reset();
	
	if (!buf.netread(sock))
	    break;

	if (sock == backendfd() && modify_serverheaders) {
	    debugmsg("First back end request seen, applying modifications\n");
	    modify_serverheaders = false;
	    while (! buf.headersreceived())
		if (!buf.netread (sock, config.backend_timeout()))
		    throw static_cast<Error>
			("Failed to get headers from back end");
	    if (config.addxrversion())
		buf.setheader("XR", VER);
	    if (config.stickyhttp() && !issticky()) {
		ostringstream o;
		o << "XRTarget=" << targetbackend();
		buf.setheader("Set-Cookie", o.str());
	    }
	}

	// Flush info to the other connected side.
	int othersock, timeout;
	if (sock == clientfd()) {
	    othersock = backendfd();
	    timeout = config.backend_timeout();
	} else {
	    othersock = clientfd();
	    timeout = config.client_timeout();
	}

	debugmsg (Mstr("Had data on ") + sock +
		  (Mstr(", sending to ") + othersock) + "\n");

	buf.netwrite(othersock, timeout);
	if (sock == backendfd())
	    balancer.backend(targetbackend()).addbytes(buf.bufsz());
    }
}
