#include "sys"
#include "error/error"
#include "profiler/profiler"
#include "config/config"

int serversocket (string addr, int port, string desc) {
    PROFILE("serversocket");
    
    int sock;

    // Create the server socket, set options
    if ( (sock = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	throw static_cast<Error>("Failed to create ") + desc + " socket: " +
	    strerror(errno);    
    int val = 1;
    if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)))
	throw static_cast<Error>("Failed to set socket options for ") +
	    desc + ": " + strerror(errno);

    // Prepare binding
    struct sockaddr_in saddr;
    memset (&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(port);

    // Assign interface to listen to
    if (addr[0] != '0') {
	msg ("Binding balancer to specific IP address " + addr + "\n");
	if ( (saddr.sin_addr.s_addr = inet_addr (addr.c_str())) == INADDR_NONE )
	    throw static_cast<Error>("Cannot convert ") + desc + " IP '" +
		addr + "' to network bytes";
    } else {
	msg ("Binding balancer to all local IP addresses\n");
	saddr.sin_addr.s_addr = htonl (INADDR_ANY);
    }
    
    // Bind and listen
    if (bind (sock, (sockaddr*) &saddr, sizeof(saddr)) < 0)
	throw static_cast<Error>("Failed to bind ") + desc +
	    " to IP/port: " + strerror(errno);
    if (listen (sock, 5) < 0)
	throw static_cast<Error>("Failed to listen to ") + desc +
	    " IP/port: " + strerror(errno);

    msg ("TCP server for " + desc + " listening\n");

    return (sock);
}
